/*
   ZoranIO.h

   ZORAN ZR36120 module driver.
   Copyright (C) 1997  Vincent Labie (try ivsparis@pratique.fr with
				      for Vincent Labie in the suject)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef	ZORANIO_H
#define	ZORANIO_H

/**********************************************************************************************************************/

#include	"Zoran.h"

/**********************************************************************************************************************/

#define	ZR_HVIDEOCONTROL_ADR			0x000
typedef	struct
	{
	unsigned	HEnd		: 10,	//  9: 0 [ 0x3ff] RW horizontal end offset from HSYNC
			HStart		: 10,	// 19:10 [ 0x001] RW horizontal start offset from HSYNC
					: 10,	// 29:20 [ 0x000] R  reserved
			HSPol		:  1,	// 30:30 [    0b] RW 0=HSYNC polarity positive
					:  1;	// 31:31 [    0b] R  reserved
	}	HVIDEOCONTROL;

#define	ZR_VVIDEOCONTROL_ADR			0x004
typedef	struct
	{
	unsigned	VEnd		: 10,	//  9: 0 [ 0x3ff] RW vertical end offset from VSYNC
			VStart		: 10,	// 19:10 [ 0x001] RW vertical start offset from VSYNC
					: 10,	// 29:20 [ 0x000] R  reserved
			VSPol		:  1,	// 30:30 [    0b] RW 0=VSYNC polarity positive
					:  1;	// 31:31 [    0b] R  reserved
	}	VVIDEOCONTROL;

#define	ZR_OUTVIDEOCONTROL_ADR			0x008
typedef	struct
	{
	unsigned	LittleEndian	:  1,	//  0: 0 [    1b] RW 1=Little Endian Pixel Format
			Pack24		:  1,	//  1: 1 [    0b] RW 0=RGB 24 bits 3 PCI cycle, 1=1DWORD
			ErrDif		:  1,	//  2: 2 [    0b] RW Error Diffusion 1=active for 555/565
			YUV2RGB		:  2,	//  4: 3 [   10b] RW 0=YUV 01=RGB888 10=RGB565 11=RGB555
					:  1,	//  5: 5 [    0b] R  reserved
			DispMod		:  1,	//  6: 6 [    0b] RW Display Mode.
						//		     0=emulated interlaced video 1=single field
					:  1,	//  7: 7 [    0b] R  reserved
			VerDcm		:  6,	// 13: 8 [  0x00] RW Vertical Decimation Ratio. per 64 dropped pixel
			HorDcm		:  6,	// 19:14 [  0x00] RW Horizontal Decimation Ratio. per 64 dropped pixel
			DupFld		:  1,	// 20:20 [    0b] RW Duplicate Field. 1=top/bottom field duplicated
			HFilter		:  3,	// 23:21 [  000b] RW 000=no luminance, 3 tap chrominace
						// 		     001=3 tap luminance, 3 tap chrominace
						// 		     010=4 tap luminance, 4 tap chrominace
						// 		     011=5 tap luminance, 4 tap chrominace
						// 		     100=4 tap luminance, 4 tap chrominace
						// 		     101-111=no luminance, 3 tap chrominace
			VCLKPol		:  1,	// 24:24 [    0b] RW Video Clock polarity
			TopField	:  1,	// 25:25 [    1b] RW Top Field Interpretation.
						//		    if ExtFI
						//			0=FI low:top field 1=FI high=top field
						//		    if internal
						//			0=HSYNC low:top field 1=HSYNC high=top field
			ExtFI		:  1,	// 26:26 [    0b] RW External Field Indication. 0=HSYNC/VSYNC, 1=extern
					:  5;	// 31:27 [  0x00] R  reserved
	}	OUTVIDEOCONTROL;

#define	ZR_ADRTOPVIDEO_ADR			0x00c
typedef	u32	ADRTOPVIDEO;		// bits 0:1 hardwired to 0

#define	ZR_ADRBOTTOMVIDEO_ADR			0x010
typedef	u32	ADRBOTTOMVIDEO;		// bits 0:1 hardwired to 0

#define	ZR_STRIDEGRABCONTROL_ADR		0x014
typedef	struct
	{
	unsigned	FrameGrab	:  1,	//  0: 0 [    0b] RS Frame Grabbing Command/Status
			SnapShot	:  1,	//  1: 1 [    0b] RW 1=Frame Grabbing Mode 0=Continuous Mode
					:  6,	//  7: 2 [  0x00] R  reserved
			VidOvf		:  1,	//  8: 8 [    0b] RC 1=Internal pixel buffer overflow
						//		     cleared by writing 1 in it
					:  7,	// 15: 9 [  0x00] R  reserved
			DispStride	: 16;	// 15: 9 [0xFFFC] R  Display Stride. incr in bytes to next line.
						//		     bits 0:1 hardwired to 0
	}	STRIDEGRABCONTROL;

#define	ZR_VIDEOCONTROL_ADR			0x018
typedef	struct
	{
	unsigned	VidWinWid	: 10,	//  9: 0 [ 0x3ff] RW Video Window Width in pixel
					:  2,	// 11:10 [   00b] R  reserved
			VidWinHt	: 10,	// 21:12 [ 0x0f0] RW Video Window Height in lines
						//		     if DispMod=0 VidWinHt is half the number
					:  2,	// 23:22 [   00b] R  reserved
			TriCom		:  1,	// 24:24 [    ?b] RW Triton Compatibility. 1 if 82437FX
						//		     what does intel messed up? :)
			MinPix		:  6,	// 30:25 [  0x07] RW Pixel FIFO Threshold, 0x01 to 0x1e
			VidEn		:  1;	// 31:31 [    1b] RW Video Enable.
	}	VIDEOCONTROL;

#define	ZR_ADRTOPVIDEOMASKMAP_ADR		0x01c
typedef	u32	ADRTOPVIDEOMASKMAP;	// bits 0:1 hardwired to 0, top field hardware mask table addr

#define	ZR_ADRBOTTOMVIDEODISPLAY_ADR		0x020
typedef	u32	ADRBOTTOMVIDEOMASKMAP;	// bits 0:1 hardwired to 0, bottom field hardware mask table addr

#define	ZR_OVERLAYCONTROL_ADR			0x024
typedef	struct
	{
	unsigned	MaskStride	:  8,	//  7: 0 [  0xff] RW MaskStride. incr in DWORD to the next line
					:  7,	// 14: 8 [  0x00] R  reserved
			OvlEnable	:  1,	// 15:15 [    0b] RW OverLay Enable
					: 16;	// 31:16 [0x0000] R  reserved
	}	OVERLAYCONTROL;

#define	ZR_SYSPCICONTROL_ADR			0x028
typedef	struct
	{
	unsigned	GenPurDir	:  8,	//  7: 0 [  0xff] RW General Purpose Pins Direction 1=input 0=ouput
					:  8,	// 15: 8 [  0x00] R  reserved
			WaitState	:  3,	// 18:16 [  000b] RW PCI Wait State Control
					:  5,	// 23:19 [  0x00] R  reserved
			SoftReset	:  1,	// 24:24 [  000b] RW Software Reset 0= reset all register to default.
					:  7;	// 31:25 [  0x00] R  reserved
	}	SYSPCICONTROL;

#define	ZR_GENPURPOSE_ADR			0x02c
typedef	struct
	{
	unsigned	RecTime0	:  2,	//  1: 0 [   00b] RW Recovery Time for Guest 0
						//		     00b 3 PCI CLK 01b 4 PCI CLK
						//		     10b 12 PCI CLK 11b 15 PCI CLK
			DurTime0	:  2,	//  3: 2 [   00b] RW Duration Time for Guest 0 same as above
			RecTime1	:  2,	//  5: 4 [   00b] RW Recovery Time for Guest 1 same as above
			DurTime1	:  2,	//  7: 6 [   00b] RW Duration Time for Guest 1 same as above
			RecTime2	:  2,	//  9: 8 [   00b] RW Recovery Time for Guest 1 same as above
			DurTime2	:  2,	// 11:10 [   00b] RW Duration Time for Guest 1 same as above
			RecTime3	:  2,	// 13:12 [   00b] RW Recovery Time for Guest 1 same as above
			DurTime3	:  2,	// 15:14 [   00b] RW Duration Time for Guest 1 same as above
					:  8,	// 23:16 [  0x00] R  reserved
			GenPurIO	:  8;	// 31:24 [  0xf0] RW General Purpose IO port
	}	GENPURPOSE;

#define	ZR_CODESOURCE_ADR			0x030
typedef	u32	CODESOURCE;		// bits 0:1 hardwired to 0

#define	ZR_CODETRANSFERCONTROLE_ADR		0x034
typedef	u32	CODETRANSFERCONTROL;	// i have nothing to do with this one, so...

#define	ZR_CODEMEMORY_ADR			0x038
typedef	u32	CODEMEMORY;		// i have nothing to do with this one, so...

#define	ZR_IRQSTATUS_ADR			0x03c
typedef	struct
	{
	unsigned			: 28,	// 27: 0 [0x0000] R  reserved
			CodRepIRQ	:  1,	// 28:28 [    0b] RC Code Report Step IRQ
			GIRQ0		:  1,	// 29:29 [    0b] RC GIRQ0 IRQ
			GIRQ1		:  1,	// 30:30 [    0b] RC GIRQ1 IRQ
					:  1;	// 31:31 [    0b] R  reserved
	}	IRQSTATUS;

#define	ZR_IRQCONTROL_ADR			0x040
typedef	struct
	{
	unsigned			: 24,	// 23: 0 [0x0000] R  reserved
			InPinEn		:  1,	// 24:24 [    0b] RW INTA# Pin Enable
					:  3,	// 27:25 [  000b] R  reserved
			CodRepIRQEn	:  1,	// 28:28 [    0b] RW Code Report Step IRQ Enable
			GIRQ0En		:  1,	// 29:29 [    0b] RC GIRQ0 Enable
			GIRQ1En		:  1,	// 30:30 [    0b] RC GIRQ1 Enable
					:  1;	// 31:31 [    0b] R  reserved
	}	IRQCONTROL;

#define	ZR_I2CCONTROL_ADR			0x044
typedef	struct
	{
	unsigned	SCL		:  1,	//  0: 0 [    1b] RW I2C SCL Line
			SDA		:  1,	//  1: 1 [    0b] RW I2C SDA Line
					: 30;	// 31: 2 [0x0000] R  reserved
	}	I2CCONTROL;

// POSTOFFICE REGISTERS 0x200-0x2ff


/**********************************************************************************************************************/

typedef	struct
	{
	HVIDEOCONTROL			HVideoControl;			// 0x000
	VVIDEOCONTROL			VVideoControl;			// 0x004
	OUTVIDEOCONTROL			OutVideoControl;		// 0x008
	ADRTOPVIDEO			AdrTopVideo;			// 0x00c
	ADRBOTTOMVIDEO			AdrBottomVideo;			// 0x010
	STRIDEGRABCONTROL		StrideGrabControl;		// 0x014
	VIDEOCONTROL			VideoControl;			// 0x018
	ADRTOPVIDEOMASKMAP		AdrTopVideoMaskMap;		// 0x01c
	ADRBOTTOMVIDEOMASKMAP		AdrBottomVideoMaskMap;		// 0x020
	OVERLAYCONTROL			OverlayControl;			// 0x024
	SYSPCICONTROL			SysPCIControl;			// 0x028
	GENPURPOSE			GenPurpose;			// 0x02c
	CODESOURCE			CodeSource;			// 0x030
	CODETRANSFERCONTROL		CodeTransferControl;		// 0x034
	CODEMEMORY			CodeMemory;			// 0x038
	IRQSTATUS			IRQStatus;			// 0x03c
	IRQCONTROL			IRQControl;			// 0x040
	I2CCONTROL			I2CControl;			// 0x044
	}	ZR36120;


/**********************************************************************************************************************/

	BOOL	ZoranReset(const ZORAN * pZoran);
	BOOL	ZoranUnreset(const ZORAN * pZoran);
	int	ZoranCapParams(ZORAN * pZoran,ZR_CAP_PARAMS * pZRCapParams);
	int	ZoranSetMode(ZORAN * pZoran,int Command);
	int	ZoranCapIsGrab(ZORAN * pZoran);
	int	ZoranCapNewGrab(ZORAN * pZoran);
	int	ZoranCapNewUser(ZORAN * pZoran);
	void	ZoranIrqProc(int Irq,void * DeviceId,struct pt_regs * fake);

/**********************************************************************************************************************/
#endif	//ZORANIO_H
